﻿using Microsoft.VisualBasic;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BasicSpectrometerConsole
{
    public class ZioLinkProtocol
    {
        public ZioLinkProtocol(string devicePath)
        {
            this.usb = new WinUsbInterface(devicePath);
        }

        public virtual void Open()
        {
            usb.Open();
        }

        public virtual void Close()
        {
            usb.Close();
        }

        public virtual bool IsOpen { get { return usb.IsOpen; } }

        private WinUsbInterface usb;
        protected byte[] replyBuffer = new byte[16492];

        //Send a command code to the device, evaluate the return code and return any other received data
        public virtual byte[] SendReceiveMessage(int CommandCode)
        {
            return SendReceiveMessage(BitConverter.GetBytes(CommandCode)); //note: command code is always sent as 4 bytes
        }

        //Send a command code and a parameter to the device, evaluate the return code and return any other received data
        public virtual byte[] SendReceiveMessage(int CommandCode, int Parameter)
        {
            byte[] sendMessage = new byte[8];
            Buffer.BlockCopy(BitConverter.GetBytes(CommandCode), 0, sendMessage, 0, 4); //copy command code to position 0
            Buffer.BlockCopy(BitConverter.GetBytes(Parameter), 0, sendMessage, 4, 4); //copy parameter to position 4
            return SendReceiveMessage(sendMessage);
        }

        //Send a byte array to the device, evaluate the return code and return any other received data
        public virtual byte[] SendReceiveMessage(byte[] sendMessage) //returns received message without return code
        {
            usb.Write(sendMessage, sendMessage.Length);
            int length = usb.Read(replyBuffer, replyBuffer.Length);
            if (length == 0) throw new Exception("No reply from device.");
            if (length < 4) throw new Exception("Not enough bytes received from device.");
            int retcode = BitConverter.ToInt32(replyBuffer, 0);
            switch (retcode >> 8)
            {
                case 0: break;
                case 1: throw new ArgumentException("Unknown command code");
                case 2: throw new ArgumentException("Invalid parameter sent to device");
                case 4: throw new InvalidOperationException("This operation is currently not allowed");
                case 5: throw new NotSupportedException("The device does not support this operation");
                case 6: throw new ArgumentException("Invalid passcode for this operation");
                case 7: throw new Exception("A communication error occurred");
                case 8: throw new Exception("Internal error received from device");
                case 10: throw new Exception("Cannot read from device memory");
                default: throw new Exception("Error code 0x" + retcode.ToString("X4") + " received from device");
            }
            length -= 4;
            byte[] reply = new byte[length];
            Buffer.BlockCopy(replyBuffer, 4, reply, 0, length);
            return reply;
        }
    }
}
